@@ -1,5 +1,7 @@
 
-from typing import Any, Set
+import asyncio
+
+from a_sync._typing import *
 
 
 class ASyncFlagException(ValueError):
@@ -31,7 +33,7 @@ class TooManyFlags(ASyncFlagException):
         super().__init__(err)
 
 class InvalidFlag(ASyncFlagException):
-    def __init__(self, flag: str):
+    def __init__(self, flag: Optional[str]):
         err = f"'flag' must be one of: {self.viable_flags}. You passed {flag}."
         err += "\nThis code should not be reached and likely indicates an issue with a custom subclass definition."
         super().__init__(err)
@@ -40,6 +42,9 @@ class InvalidFlagValue(ASyncFlagException):
     def __init__(self, flag: str, flag_value: Any):
         super().__init__(f"'{flag}' should be boolean. You passed {flag_value}.")
 
+class FlagNotDefined(ASyncFlagException):
+    def __init__(self, obj: Type, flag: str):
+        super().__init__(f"{obj} flag {flag} is not defined.")
 
 
 class ImproperFunctionType(ValueError):
@@ -47,7 +52,11 @@ class ImproperFunctionType(ValueError):
 
 class FunctionNotAsync(ImproperFunctionType):
     def __init__(self, fn):
-        super().__init__(f"'coro_fn' must be a coroutine function defined with 'async def'. You passed {fn}.")
+        super().__init__(f"`coro_fn` must be a coroutine function defined with `async def`. You passed {fn}.")
+
+class FunctionNotSync(ImproperFunctionType):
+    def __init__(self, fn):
+        super().__init__(f"`func` must be a coroutine function defined with `def`. You passed {fn}.")
 
 class KwargsUnsupportedError(ValueError):
     def __init__(self):
@@ -63,3 +72,17 @@ class SyncModeInAsyncContextError(ASyncRuntimeError):
         err = f"The event loop is already running, which means you're trying to use an ASync function synchronously from within an async context.\n"
         err += f"Check your traceback to determine which, then try calling asynchronously instead with one of the following kwargs:\n"
         err += f"{_flags.VIABLE_FLAGS}"
+
+class MappingIsEmptyError(Exception):
+    def __init__(self):
+        super().__init__("TaskMapping does not contain anything to yield")
+
+class MappingNotEmptyError(Exception):
+    def __init__(self):
+        super().__init__("TaskMapping already contains some data. In order to use `map`, you need a fresh one.")
+
+class PersistedTaskException(Exception):
+    def __init__(self, exc: E, task: asyncio.Task) -> None:
+        super().__init__(f"{exc.__class__.__name__}: {exc}", task)
+        self.exception = exc
+        self.task = task
