@@ -2,19 +2,11 @@
 
 import asyncio
 import concurrent.futures
-from decimal import Decimal
 from functools import partial, wraps
 from inspect import isawaitable
-from typing import (Any, Awaitable, Callable, Generic, List, Optional, Set,
-                    TypeVar, Union, final, overload)
 
-from typing_extensions import ParamSpec, Self, Unpack
+from a_sync._typing import *
 
-from a_sync._typing import ModifierKwargs
-
-T = TypeVar('T')
-P = ParamSpec('P')
-MaybeAwaitable = Union[T, Awaitable[T]]
 
 def future(callable: Union[Callable[P, Awaitable[T]], Callable[P, T]] = None, **kwargs: Unpack[ModifierKwargs]) -> Callable[P, Union[T, "ASyncFuture[T]"]]:
     return _ASyncFutureWrappedFn(callable, **kwargs)
@@ -31,7 +23,7 @@ def _materialize(meta: "ASyncFuture[T]") -> T:
     except RuntimeError as e:
         raise RuntimeError(f"{meta} result is not set and the event loop is running, you will need to await it first") from e
 
-Numeric = Union[int, float, Decimal, "ASyncFuture[int]", "ASyncFuture[float]", "ASyncFuture[Decimal]"]
+MetaNumeric = Union[Numeric, "ASyncFuture[int]", "ASyncFuture[float]", "ASyncFuture[Decimal]"]
 
 class ASyncFuture(concurrent.futures.Future, Awaitable[T]):
     __slots__ = "__awaitable__", "__dependencies", "__dependants", "__task"
@@ -59,11 +51,18 @@ class ASyncFuture(concurrent.futures.Future, Awaitable[T]):
         return [self]
     @property
     def result(self) -> Union[Callable[[], T], Any]:
-        if hasattr(r := super().result(), 'result'):
-            # can be property, method, whatever. should work.
-            return r.result
-        # the result should be callable like an asyncio.Future
-        return super().result
+        """
+        If this future is not done, it will work like cf.Future.result. It will block, await the awaitable, and return the result when ready.
+        If this future is done and the result has attribute `results`, will return `getattr(future_result, 'result')`
+        If this future is done and the result does NOT have attribute `results`, will again work like cf.Future.result
+        """
+        if self.done():
+            if hasattr(r := super().result(), 'result'):
+                # can be property, method, whatever. should work.
+                return r.result
+            # the result should be callable like an asyncio.Future
+            return super().result
+        return lambda: _materialize(self)
     def __getattr__(self, attr: str) -> Any:
         return getattr(_materialize(self), attr)
     def __getitem__(self, key) -> Any:
@@ -121,7 +120,7 @@ class ASyncFuture(concurrent.futures.Future, Awaitable[T]):
     def __add__(self: "ASyncFuture[Decimal]", other: Awaitable[int]) -> "ASyncFuture[Decimal]":...
     @overload
     def __add__(self: "ASyncFuture[int]", other: Awaitable[Decimal]) -> "ASyncFuture[Decimal]":...
-    def __add__(self, other: Numeric) -> "ASyncFuture":
+    def __add__(self, other: MetaNumeric) -> "ASyncFuture":
         return ASyncFuture(self.__add(other), dependencies=self.__list_dependencies(other))
     @overload
     def __sub__(self: "ASyncFuture[int]", other: int) -> "ASyncFuture[int]":...
@@ -151,7 +150,7 @@ class ASyncFuture(concurrent.futures.Future, Awaitable[T]):
     def __sub__(self: "ASyncFuture[Decimal]", other: Awaitable[int]) -> "ASyncFuture[Decimal]":...
     @overload
     def __sub__(self: "ASyncFuture[int]", other: Awaitable[Decimal]) -> "ASyncFuture[Decimal]":...
-    def __sub__(self, other: Numeric) -> "ASyncFuture":
+    def __sub__(self, other: MetaNumeric) -> "ASyncFuture":
         return ASyncFuture(self.__sub(other), dependencies=self.__list_dependencies(other))
     def __mul__(self, other) -> "ASyncFuture":
         return ASyncFuture(self.__mul(other), dependencies=self.__list_dependencies(other))
