@@ -1,68 +1,82 @@
-# mypy: disable-error-code=valid-type
-# mypy: disable-error-code=misc
+
 import functools
 import sys
 
-from a_sync import _helpers, _kwargs, exceptions
+from a_sync import _helpers, _kwargs
 from a_sync._typing import *
 from a_sync.modifiers.manager import ModifierManager
     
 
-class Modified(Generic[T]):
+class ModifiedMixin:
     modifiers: ModifierManager
+    __slots__ = "modifiers", "wrapped"
     def _asyncify(self, func: SyncFn[P, T]) -> CoroFn[P, T]:
         """Applies only async modifiers."""
         coro_fn = _helpers._asyncify(func, self.modifiers.executor)
         return self.modifiers.apply_async_modifiers(coro_fn)
     @functools.cached_property
-    def _await(self): # -> SyncFn[[Awaitable[T]]] T]:
+    def _await(self) -> Callable[[Awaitable[T]], T]:
         """Applies only sync modifiers."""
         return self.modifiers.apply_sync_modifiers(_helpers._await)
-    @property
+    @functools.cached_property
     def default(self) -> DefaultMode:
         return self.modifiers.default
     
         
-class ASyncFunction(Modified[T], Callable[P, T]):
+class ASyncFunction(ModifiedMixin, Generic[P, T]):
+    # NOTE: We can't use __slots__ here because it breaks functools.update_wrapper
     @overload
     def __init__(self, fn: CoroFn[P, T], **modifiers: Unpack[ModifierKwargs]) -> None:...
     @overload
     def __init__(self, fn: SyncFn[P, T], **modifiers: Unpack[ModifierKwargs]) -> None:...
     def __init__(self, fn: AnyFn[P, T], **modifiers: Unpack[ModifierKwargs]) -> None:
         _helpers._validate_wrapped_fn(fn)
-        self.modifiers = ModifierManager(**modifiers)
-        self._fn = fn
+        self.modifiers = ModifierManager(modifiers)
+        self.__wrapped__ = fn
+        functools.update_wrapper(self, self.__wrapped__)
     
+    @overload
+    def __call__(self, *args: P.args, sync: Literal[True], **kwargs: P.kwargs) -> T:...
+    @overload
+    def __call__(self, *args: P.args, sync: Literal[False], **kwargs: P.kwargs) -> Awaitable[T]:...
+    @overload
+    def __call__(self, *args: P.args, asynchronous: Literal[False], **kwargs: P.kwargs) -> T:...
+    @overload
+    def __call__(self, *args: P.args, asynchronous: Literal[True], **kwargs: P.kwargs) -> Awaitable[T]:...
     def __call__(self, *args: P.args, **kwargs: P.kwargs) -> MaybeAwaitable[T]:
         return self.fn(*args, **kwargs)
     
+    def __repr__(self) -> str:
+        return f"<{self.__class__.__name__} {self.__module__}.{self.__name__} at {hex(id(self))}>"
+    
     @functools.cached_property
     def fn(self): # -> Union[SyncFn[[CoroFn[P, T]], MaybeAwaitable[T]], SyncFn[[SyncFn[P, T]], MaybeAwaitable[T]]]:
         """Returns the final wrapped version of 'self._fn' decorated with all of the a_sync goodness."""
         return self._async_wrap if self._async_def else self._sync_wrap
     
-    @property
+    @functools.cached_property
     def _sync_default(self) -> bool:
         """If user did not specify a default, we defer to the function. 'def' vs 'async def'"""
         return True if self.default == 'sync' else False if self.default == 'async' else not self._async_def
     
     @property
     def _async_def(self) -> bool:
-        return asyncio.iscoroutinefunction(self._fn)
+        return asyncio.iscoroutinefunction(self.__wrapped__)
     
-    def _run_sync(self, kwargs: dict):
-        try:
+    def _run_sync(self, kwargs: dict) -> bool:
+        if flag := _kwargs.get_flag_name(kwargs):
             # If a flag was specified in the kwargs, we will defer to it.
-            return _kwargs.is_sync(kwargs, pop_flag=True)
-        except exceptions.NoFlagsFound:
+            return _kwargs.is_sync(flag, kwargs, pop_flag=True)
+        else:
             # No flag specified in the kwargs, we will defer to 'default'.
             return self._sync_default
     
     @functools.cached_property
     def _asyncified(self) -> CoroFn[P, T]:
         """Turns 'self._fn' async and applies both sync and async modifiers."""
-        assert not self._async_def, f"Can only be applied to sync functions, not {self._fn}"
-        return self._asyncify(self._modified_fn)
+        if self._async_def:
+            raise TypeError(f"Can only be applied to sync functions, not {self.__wrapped__}")
+        return self._asyncify(self._modified_fn)  # type: ignore [arg-type]
     
     @functools.cached_property
     def _modified_fn(self) -> AnyFn[P, T]:
@@ -71,8 +85,8 @@ class ASyncFunction(Modified[T], Callable[P, T]):
         Applies async modifiers to 'self._fn' if 'self._fn' is a sync function.
         """
         if self._async_def:
-            return self.modifiers.apply_async_modifiers(self._fn)
-        return self.modifiers.apply_sync_modifiers(self._fn)
+            return self.modifiers.apply_async_modifiers(self.__wrapped__)  # type: ignore [arg-type]
+        return self.modifiers.apply_sync_modifiers(self.__wrapped__)  # type: ignore [return-value]
     
     @functools.cached_property
     def _async_wrap(self): # -> SyncFn[[CoroFn[P, T]], MaybeAwaitable[T]]:
@@ -93,22 +107,74 @@ class ASyncFunction(Modified[T], Callable[P, T]):
                 return self._modified_fn(*args, **kwargs)
             return self._asyncified(*args, **kwargs)            
         return sync_wrap
-    
+
 if sys.version_info < (3, 10):
     _inherit = ASyncFunction[AnyFn[P, T], ASyncFunction[P, T]]
 else:
     _inherit = ASyncFunction[[AnyFn[P, T]], ASyncFunction[P, T]]
               
-class ASyncDecorator(_inherit):
-    _fn = None
+class ASyncDecorator(ModifiedMixin):
     def __init__(self, **modifiers: Unpack[ModifierKwargs]) -> None:
         assert 'default' in modifiers, modifiers
-        self.modifiers = ModifierManager(**modifiers)
+        self.modifiers = ModifierManager(modifiers)
         self.validate_inputs()
         
     def validate_inputs(self) -> None:
-        if self.default not in ['sync', 'async', None]:
-            raise ValueError(f"'default' must be either 'sync', 'async', or None. You passed {self.default}.")
+        if self.modifiers.default not in ['sync', 'async', None]:
+            raise ValueError(f"'default' must be either 'sync', 'async', or None. You passed {self.modifiers.default}.")
         
-    def __call__(self, func: AnyFn[P, T]) -> ASyncFunction[P, T]:
+    @overload
+    def __call__(self, func: CoroFn[P, T]) -> ASyncFunction[P, T]:  # type: ignore [override]
+        ...
+    @overload
+    def __call__(self, func: SyncFn[P, T]) -> ASyncFunction[P, T]:  # type: ignore [override]
+        ...
+    def __call__(self, func: AnyFn[P, T]) -> ASyncFunction[P, T]:  # type: ignore [override]
         return ASyncFunction(func, **self.modifiers)
+
+
+# Mypy helper classes
+
+class ASyncFunctionSyncDefault(ASyncFunction[P, T]):
+    @overload
+    def __call__(self, *args: P.args, sync: Literal[True], **kwargs: P.kwargs) -> T:...
+    @overload
+    def __call__(self, *args: P.args, sync: Literal[False], **kwargs: P.kwargs) -> Awaitable[T]:...
+    @overload
+    def __call__(self, *args: P.args, asynchronous: Literal[False], **kwargs: P.kwargs) -> T:...
+    @overload
+    def __call__(self, *args: P.args, asynchronous: Literal[True], **kwargs: P.kwargs) -> Awaitable[T]:...
+    @overload
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> T:...
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> MaybeAwaitable[T]:
+        return self.fn(*args, **kwargs)
+    
+class ASyncFunctionAsyncDefault(ASyncFunction[P, T]):
+    @overload
+    def __call__(self, *args: P.args, sync: Literal[True], **kwargs: P.kwargs) -> T:...
+    @overload
+    def __call__(self, *args: P.args, sync: Literal[False], **kwargs: P.kwargs) -> Awaitable[T]:...
+    @overload
+    def __call__(self, *args: P.args, asynchronous: Literal[False], **kwargs: P.kwargs) -> T:...
+    @overload
+    def __call__(self, *args: P.args, asynchronous: Literal[True], **kwargs: P.kwargs) -> Awaitable[T]:...
+    @overload
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> Awaitable[T]:...
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> MaybeAwaitable[T]:
+        return self.fn(*args, **kwargs)
+
+class ASyncDecoratorSyncDefault(ASyncDecorator):
+    @overload
+    def __call__(self, func: AnyBoundMethod[P, T]) -> ASyncFunctionSyncDefault[P, T]:  # type: ignore [override]
+        ...
+    @overload
+    def __call__(self, func: AnyFn[P, T]) -> ASyncFunctionSyncDefault[P, T]:  # type: ignore [override]
+        ...
+
+class ASyncDecoratorAsyncDefault(ASyncDecorator):
+    @overload
+    def __call__(self, func: AnyBoundMethod[P, T]) -> ASyncFunctionAsyncDefault[P, T]:  # type: ignore [override]
+        ...
+    @overload
+    def __call__(self, func: AnyFn[P, T]) -> ASyncFunctionAsyncDefault[P, T]:  # type: ignore [override]
+        ...
