@@ -17,8 +17,8 @@ class ModifierManager(Dict[str, Any]):
     semaphore: SemaphoreSpec
     # sync modifiers
     executor: Executor
-
-    def __init__(self, **modifiers: ModifierKwargs) -> None:
+    __slots__ = "_modifiers", 
+    def __init__(self, modifiers: ModifierKwargs) -> None:
         for key in modifiers.keys():
             if key not in valid_modifiers:
                 raise ValueError(f"'{key}' is not a supported modifier.")
@@ -70,13 +70,13 @@ class ModifierManager(Dict[str, Any]):
         return sync_modifier_wrap
     
     # Dictionary api
-    def items(self) -> List[Tuple[str, Any]]:
-        return self._modifiers.items()
-    def keys(self) -> List[str]:
+    def keys(self) -> KeysView[str]:  # type: ignore [override]
         return self._modifiers.keys()
-    def values(self) -> List[Any]:
+    def values(self) -> ValuesView[Any]:  # type: ignore [override]
         return self._modifiers.values()
-    def __contains__(self, key: str) -> bool:
+    def items(self) -> ItemsView[str, Any]:  # type: ignore [override]
+        return self._modifiers.items()
+    def __contains__(self, key: str) -> bool:  # type: ignore [override]
         return key in self._modifiers
     def __iter__(self) -> Iterator[str]:
         return self._modifiers.__iter__()
@@ -85,5 +85,5 @@ class ModifierManager(Dict[str, Any]):
     def __getitem__(self, modifier_key: str):
         return self._modifiers[modifier_key]  # type: ignore [literal-required]
 
-nulls = ModifierManager(**null_modifiers)
-user_defaults = ModifierManager(**user_set_default_modifiers)
+nulls = ModifierManager(null_modifiers)
+user_defaults = ModifierManager(user_set_default_modifiers)
