@@ -4,8 +4,9 @@ import time
 
 import pytest
 
+from a_sync._bound import ASyncBoundMethodAsyncDefault
 from a_sync._meta import ASyncMeta
-from tests.fixtures import TestClass, TestInheritor, TestMeta, increment, TestSync
+from tests.fixtures import TestClass, TestInheritor, TestMeta, increment, TestSync, WrongThreadError
 
 classes = pytest.mark.parametrize('cls', [TestClass, TestSync, TestInheritor, TestMeta])
 
@@ -24,13 +25,26 @@ def test_base_sync(cls: type, i: int):
     assert duration < 3, "There is a 2 second sleep in 'test_cached_property' but it should only run once."
 
     # Can we override with kwargs?
-    val = asyncio.get_event_loop().run_until_complete(sync_instance.test_fn(sync=False))
-    assert isinstance(val, int)
+    if isinstance(sync_instance, TestSync):
+        with pytest.raises(WrongThreadError):
+            # this raises an assertion error inside of the test_fn execution. this is okay.
+            asyncio.get_event_loop().run_until_complete(sync_instance.test_fn(sync=False))
+    else:
+        val = asyncio.get_event_loop().run_until_complete(sync_instance.test_fn(sync=False))
+        assert isinstance(val, int)
 
     # Can we access hidden methods for properties?
-    assert sync_instance.__test_property__() == i * 2
+    getter = sync_instance.__test_property__
+    assert isinstance(getter, ASyncBoundMethodAsyncDefault), getter
+    getter_coro = getter()
+    assert asyncio.iscoroutine(getter_coro), getter_coro
+    assert asyncio.get_event_loop().run_until_complete(getter_coro) == i * 2
     start = time.time()
-    assert sync_instance.__test_cached_property__() == i * 3
+    getter = sync_instance.__test_cached_property__
+    assert isinstance(getter, ASyncBoundMethodAsyncDefault), getter
+    getter_coro = getter()
+    assert asyncio.iscoroutine(getter_coro), getter_coro
+    assert asyncio.get_event_loop().run_until_complete(getter_coro) == i * 3
     # Can we override them too?
     assert asyncio.get_event_loop().run_until_complete(sync_instance.__test_cached_property__(sync=False)) == i * 3
     duration = time.time() - start
@@ -43,7 +57,11 @@ async def test_base_async(cls: type, i: int):
     async_instance = cls(i, False)
     assert isinstance(async_instance.__class__, ASyncMeta)
     
-    assert await async_instance.test_fn() == i
+    if isinstance(async_instance, TestSync):
+        with pytest.raises(WrongThreadError):
+            assert await async_instance.test_fn() == i
+    else:
+        assert await async_instance.test_fn() == i
     assert await async_instance.test_property == i * 2
     start = time.time()
     assert await async_instance.test_cached_property == i * 3
@@ -65,8 +83,17 @@ async def test_base_async(cls: type, i: int):
             async_instance.test_fn(sync=True)
     
     # Can we access hidden methods for properties?
-    assert await async_instance.__test_property__() == i * 2
-    assert await async_instance.__test_cached_property__() == i * 3
+    getter = async_instance.__test_property__
+    assert isinstance(getter, ASyncBoundMethodAsyncDefault), getter
+    getter_coro = getter()
+    assert asyncio.iscoroutine(getter_coro), getter_coro
+    assert await getter_coro == i * 2
+
+    getter = async_instance.__test_cached_property__
+    assert isinstance(getter, ASyncBoundMethodAsyncDefault), getter
+    getter_coro = getter()
+    assert asyncio.iscoroutine(getter_coro), getter_coro
+    assert await getter_coro == i * 3
     # Can we override them too?
     with pytest.raises(RuntimeError):
         async_instance.__test_cached_property__(sync=True)
